% Find the "optimal" number of communities given a network using an eigenvector method
% Source: MEJ Newman: Finding community structure using the eigenvectors of matrices, arXiv:physics/0605087
% Newman, "Modularity and community structure in networks", arxiv.org/pdf/physics/0602124v1
% Q=(s^T)Bs, Bij=Aij-kikj/2m
% Bij^g=Bij - delta_ij * (sum k over g)B_ik
% Bij^g=(Aij-kikj/2m)-delta_ij(sum k over g)(A(g)_ik-deg(g)_i deg(k)_j/2(m_g))
% Bij^g=(Aij-kikj/2m)-delta_ij(k_i^(g)-k_i*sum(deg^(g)/2m)
% STEPS:
% 1 define current modularity matrix
% 2 compute eigenvector corresp. to largest eigenvalue
% 3 separate into 2 modules based on signs in eigenvector
% terminate when max eigenvalue is 0 for all subgraphs
% 
% Other functions used: numedges.m, degrees.m, subgraph.m, isconnected.m
% GB, Last modified: November 17, 2009
%modified by Meng Dec1,2021
% Copyright (c) 2011, Massachusetts Institute of Technology.
% All rights reserved.
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
% 
% - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
% - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
% - Neither the name of the Massachusetts Institute of Technology nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
% 
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

function modules=newman_eigenvector_method_modified_sparse_computation(adj)

modules={};
n=length(adj);
m=numedges(adj);
[deg,~,~]=degrees(adj);
queue{1}=[1:n];   % append all nodes to the queue
deltaQ=inf;
% tmp=adj * sparse(ones(n,1));

% B=adj-tmp*tmp'/(2*m);

% B = @(v)Mv(v, adj);

while length(queue)>0  % while there is always a divisible subgraph
    
    % compute modularity matrix- Bg
    G=queue{1};            % nodes in current (sub)graph to partition
    hat_Bg = @(v)Mgv(v, adj, deg', 2*m, G);
%     hat_Bg_matrix = hat_Bg(eye(length(G)))
%     row_sum=hat_Bg_matrix*ones(length(G),1)
%     [V,E]=eigs(@(v)Mgv(v, adj, deg', 2*m, G), length(G));
% %     [V,E]=eigs(hat_Bg_matrix, length(G));
    [V,E]=eigs(hat_Bg, length(G), eye(length(G)), 1, 'largestreal', 'IsFunctionSymmetric' , 1);
%     diag(E)


%     if  abs(max(diag(E)))<=10^(-5) % terminate - indivisible
    if  (max(diag(E)))<=10^(-5) % terminate - indivisible
        queue=queue(2:length(queue));
        modules=[modules G];
        [~,p]=size(modules);
        continue
    end
    
    [max_eigv,indmax]=max(diag(E));
%     max_eigv
%     indmax
    u1=V(:,indmax);
    comm1=find(u1>0); comm2=find(u1<=0);  % indices in G
    
          % Get deltaQ
    nBg=length(G);
    s=zeros(nBg, 1);
    for i=1:nBg
        if u1(i)>0
            s(i)=1;
        else
            s(i)=-1;
        end    
    end
    
    tmp = hat_Bg(s);  
    deltaQ=s'*tmp;
        
    if deltaQ<10^(-5)
        queue=queue(2:length(queue));   % remove(G)
        modules=[modules G];            % keep original G as indivisible
        continue  
    end
%     comm1
    
% %     if not(isconnected(adj(G(comm1),G(comm1)))) | not(isconnected(adj(G(comm2),G(comm2)))) 
%         queue=queue(2:length(queue));   % remove(G)
%         modules=[modules G]            % keep original G as indivisible
%       %'found disconnected subgraphs - do not explore this option    
%       continue  
%     end
%     comm1
    queue=[queue G(comm1) G(comm2)];
    queue=queue(2:length(queue));
%     else
%         return
end   
end

function output = Mgv(v, A, degA, SallA, G)
    A = A(G, G);
    tmp = degA(G);
    n = size(A, 1);
    c = (tmp'*ones(n,1))/SallA;
    output = A * v - tmp * ((tmp' * v) / SallA)-(A*ones(n,1) - c*tmp).*v;
end

